// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// Updates the .xml files by deleting existing files and 
// creating them again from the .sci with help_from_sci.

function b = isContentUptodate ( content , filename )
  // Check if a content matches a file.
  // 
  // Calling Sequence
  //   b = isContentUptodate ( content , filename )
  //
  // Parameters
  //   content : a 1x1 matrix of strings, the new content
  //   filename : a 1x1 matrix of strings, the file to be checked.
  //   b : a 1x1 matrix of booleans
  //
  // Description
  //   Returns true if the <literal>filename</literal> is up-to-date, that is,
  //   if the file does not require to be changed.
  //   The file is to be updated if one of the following conditions is satisfied.
  // <itemizedlist>
  //   <listitem>
  //     The file does not exist.
  //   </listitem>
  //   <listitem>
  //     The file exists, but its content is not equal to <literal>content</literal>.
  //     The content comparison ignores the leading and trailing blanks.
  //   </listitem>
  // </itemizedlist>
  //   Last update : 02/09/2010.
  //
  // Example
  //   // TODO...
  //
  // Author
  //   2010 - DIGITEO - Michael Baudin
  //   
  
  b = %t
  if ( fileinfo(filename) == [] ) then
    b = %f
  else
    txt = mgetl(filename)
    if ( or ( stripblanks(content) <> stripblanks(txt) ) ) then
      b = %f
    end
  end
endfunction

function b = fileUpdateIfNeeded ( content , filename )
  // Update a file if this is needed.
  //
  // Calling Sequence
  //   b = fileUpdateIfNeeded ( content , filename )
  //
  // Parameters
  //   content : a 1x1 matrix of strings, the new content
  //   filename : a 1x1 matrix of strings, the file to be checked.
  //   b : a 1x1 matrix of booleans
  //
  // Description
  //   Returns true if the <literal>filename</literal> is up-to-date, that is,
  //   if the file was not changed.
  //   Returns false if the file was changed.
  //   The file is to be updated according to the rules of <literal>isContentUptodate</literal>.
  //   Generates an error if the file was to be changed, but that was not possible.
  //   Last update : 02/09/2010.
  //
  // Example
  //   // TODO...
  //
  // Author
  //   2010 - DIGITEO - Michael Baudin
  //   

  b = isContentUptodate ( content , filename )
  if ( ~b ) then
    r = mputl ( content , filename )
    if ( ~r ) then
      error(sprintf(gettext("%s: Unable to write xml file: %s\n"),"fileUpdateIfNeeded",filename));
    end
  end
endfunction

function helpupdate ( funarray , helpdir , macrosdir , demosdir , modulename , verbose )
  // Update the help and the demos from the .sci files.
  //
  // Calling Sequence
  //   updatehelp ( funarray , helpdir , macrosdir , demosdir , modulename , verbose )
  //
  // Parameters
  //   funarray : column matrix of strings. The list of functions to update
  //   helpdir : the help directory
  //   macrosdir : the macros directory
  //   demosdir : the demonstration directory
  //   modulename: a 1x1 matrix of strings, the name of the module to update
  //   verbose: a 1x1 matrix of booleans, verbose = %t prints messages
  //   modulename : the name of the module
  //
  // Description
  //   Update the .xml help files and the demos scripts
  //   from the macros corresponding to the function array
  //   of strings funarray.
  //   The existing .xml files in the help dir which 
  //   correspond to file in the funarray are deleted (Caution !).
  //   Generates the .xml and the .sce files from the help_from_sci function.
  //   Generates a demonstration gateway.
  //
  //   If demosdir is an empty matrix, do not generate the demonstrations.
  //   Last update : 04/10/2010.
  //
  // Author
  //   2010, Michael Baudin
  
  if ( fileinfo ( helpdir ) == [] ) then
    error(sprintf(gettext("%s: Wrong help directory: %s does not exist.\n"),"updatehelp",helpdir));
  end
  if ( fileinfo ( macrosdir ) == [] ) then
    error(sprintf(gettext("%s: Wrong macros directory: %s does not exist.\n"),"updatehelp",macrosdir));
  end
  if ( demosdir <> [] ) then
    if ( fileinfo ( demosdir ) == [] ) then
      error(sprintf(gettext("%s: Wrong demos directory: %s does not exist.\n"),"updatehelp",demosdir));
    end
  end
  
  //
  // 2. Generate each .xml and each .sce from the .sci
  flist = ls(macrosdir)';
  for f = flist
    issci = regexp(f,"/(.*).sci/");
    kf = find(funarray==basename(f))
    if ( issci <> [] & kf <> [] ) then
      scifile = fullfile ( macrosdir , f )
      if ( verbose ) then
        mprintf("Processing %s\n",scifile);
      end
      funname = funarray(kf)
      xmlfile = fullfile ( helpdir , funname + ".xml" )
      // Generate the xml and the demo content
      [helptxt,demotxt]= help_from_sci (scifile)
      // Delete the "info" tag, containing the date (3 lines)
      k = find(stripblanks(helptxt)=="<info>")
      helptxt(k:k+2) = []
      // Update the xml file, if necessary
      isuptodate = fileUpdateIfNeeded ( helptxt , xmlfile )
      if ( ~isuptodate & verbose ) then
        changetxt = "XML Changed"
        mprintf("  %s %s\n",changetxt,xmlfile);
      end
      // Create the demo
      if ( demosdir <> [] ) then
        demofile = fullfile ( demosdir , funname + ".sce" )
        // Update the demo script
        header = []
        header($+1) = "//"
        header($+1) = "// This help file was automatically generated from "+funname+".sci using help_from_sci()."
        header($+1) = "// PLEASE DO NOT EDIT"
        header($+1) = "//"
        footer = []
        footer($+1) = msprintf("//\n");
        footer($+1) = msprintf("// Load this script into the editor\n");
        footer($+1) = msprintf("//\n");
        footer($+1) = msprintf("filename = ""%s"";\n",funname + ".sce");
        footer($+1) = msprintf("dname = get_absolute_file_path(filename);\n");
        footer($+1) = msprintf("editor ( fullfile(dname,filename) );\n");
        demotxt = [header;demotxt;footer]
        // Update the demo file, if necessary
        isuptodate = fileUpdateIfNeeded ( demotxt , demofile )
        if ( ~isuptodate & verbose ) then
          changetxt = "SCE Changed"
          mprintf("  %s %s\n",changetxt,demofile);
        end
      end
    end
  end
  //
  // 3. Generates the Demonstration gateway
  // Include all .sce files in the gateway, including 
  // handcrafted scripts which might have been written
  // by the user (and not generated by this function).
  // To make so, use the list of .sce files in the demo dir and 
  /// not simply the funarray.
  if ( demosdir <> [] ) then
    gatetxt = []
    gatetxt ($+1) = "// This help file was automatically generated using helpupdate"
    gatetxt ($+1) = "// PLEASE DO NOT EDIT"
    gatetxt ($+1) = "demopath = get_absolute_file_path(""" + modulename + ".dem.gateway.sce"");"
    gatetxt ($+1) = "subdemolist = ["
    flist = ls(demosdir)';
    for f = flist
      issce = regexp(f,"/(.*).sce/");
      isgateway = regexp(f,"/(.*).gateway.sce/");
      if ( issce <> [] & isgateway == [] ) then
        flen = length(f)
        funname = part(f,[1:flen-4])
        gatetxt($+1) = """" + funname + """, """ + funname + ".sce""; .."
      end
    end
    //
    gatetxt ($+1) = "];"
    gatetxt ($+1) = "subdemolist(:,2) = demopath + subdemolist(:,2)"
    gatefile = fullfile ( demosdir , modulename+".dem.gateway.sce" )
    // Update the gateway file, if necessary
    isuptodate = fileUpdateIfNeeded ( gatetxt , gatefile )
    if ( ~isuptodate & verbose ) then
      changetxt = "SCE Gateway Changed"
      mprintf("%s %s\n",changetxt,gatefile);
    end
  end
endfunction

//
cwd = get_absolute_file_path("update_help.sce");
mprintf("Working dir = %s\n",cwd);

//
// Generate the Pursuit help
mprintf("Updating Wavelab/Pursuit\n");
helpdir = fullfile(cwd,"Pursuit");
funmat = [
  "CPAtomicSynthesis"
  "CPImpulse"
  "CPMPursuitTour"
  "CPPursuitBF"
  "CPPursuit"
  "CPPursuitTour"
  "FCPSynthesis"
  "FWPAtomicSynthesis"
  "FWPSynthesis"
  "ImageAtomicPhase"
  "ix2pkt"
  "MakeSynthesisTable"
  "MonitorPursuit"
  "pkt2ix"
  "PlotAtomicPhase"
  "PlotSynthesisTable"
  "WPAtomicSynthesis"
  "WPImpulse"
  "WPMPursuitTour"
  "WPPursuitBF"
  "WPPursuit"
  "WPPursuitTour"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )

//
// Generate the ambiguity help
mprintf("Updating Wavelab/Utilities\n");
helpdir = fullfile(cwd,"Utilities");
funmat = [
  "CutDyad"
  "HitAnyKey"
  "LockAxes"
  "PadDyad"
  "ShapeAsRow"
  "ShapeLike"
  "UnlockAxes"
  "WaitUntil"
  "WhiteNoise"
  "log2lin"
  "rnshift"
  "AutoImage"
  "GrayImage"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//

//
// Generate the ambiguity help
mprintf("Updating Wavelab/Continuous\n");
helpdir = fullfile(cwd,"Continuous");
funmat = [
  "RWT"
  "IRWT"
  "MM_RWT"
  "ImageRWT"
  "AWT"
  "Ridge_AWT"
  "CWT"
  "CWTscale"
  "ImageCWT"
  "WTMM"
  "ImageWTMM"
  "SkelMap"
  "BuildSkelMap"
  "BuildSkelMapFast"
  "PlotSkelMap"
  "PruneSkelMap"
  "ExtractRidge"
  "PlotRidges"
  "CalcThermoPartition"
  "PlotThermoPartition"
  "CalcGenFracDimen"
  "PlotGenFracDimen"
  "CalcMomentGenFun"
  "PlotMomentGenFun"
  "CalcFracSpectrum"
  "PlotFracSpectrum"
  "CalcCWTScale"
  "CalcCWTPars"
  "CalcCWTNorms"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//


//
// Generate the ambiguity help
mprintf("Updating Wavelab/TimeFrequency\n");
helpdir = fullfile(cwd,"TimeFrequency");
funmat = [
  "WindowFT"
  "IWindowFT"
  "Ridge_WindowFT"
  "CohenDist"
  "Spectrogram"
  "WVDist"
  "WignerDist"
  "WVDist_AF"
  "TFDist_AF"
  "Interpol2"
  "MakeWindow"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//



//
// Generate the ambiguity help
mprintf("Updating Wavelab/Invariant\n");
helpdir = fullfile(cwd,"Invariant");
funmat = [
  "FWT_ATrou"
  "IWT_ATrou"
  "FWT_Stat"
  "IWT_Stat"
  "FWT_TI"
  "IWT_TI"
  "FWT2_TI"
  "IWT2_TI"
  "FWT2_Atrou"
  "BestShift"
  "CalcShiftTree"
  "UnpackShiftCoeffs"
  "PackShiftCoeffs"
  "MM_DWT"
  "IMM_DWT"
  "MM2_DWT"
  "IMM2_DWT"
  "PlotStatTable"
  "MakeDyadFilter"
  "MakeATrouFilter"
  "ModulusMaxima"
  "TI_2_Stat"
  "Stat_2_TI"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//


//
// Generate the ambiguity help
mprintf("Updating Wavelab/Fractals\n");
helpdir = fullfile(cwd,"Fractals");
funmat = [
  "FracPartition"
  "FracScalExp"
  "FracSingSpect"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//


// Generate the ambiguity help
mprintf("Updating Wavelab/Datasets\n");
helpdir = fullfile(cwd,"Datasets");
funmat = [
  "ReadImage"
  "ReadSignal"
  "MakeBrownian"
  "MakeCantor"
  "MakeFractal"
  "MakeImage"
  "MakeProcess"
  "MakeSignal"
  "makediag"
  "MAT2RAW"
  "RAW2MAT"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//



// Generate the ambiguity help
mprintf("Updating Wavelab/Biorthogonal\n");
helpdir = fullfile(cwd,"Biorthogonal");
funmat = [
  "FWT_PBS"
  "FWT_PB"
  "IWT_PBS"
  "IWT_PB"
  "FWT_SBS"
  "IWT_SBS"
  "FWT2_PBS"
  "FWT2_PB"
  "IWT2_PBS"
  "IWT2_PB"
  "FWT2_SBS"
  "IWT2_SBS"
  "PlotPBSMultiRes"
  "MakeBSFilter"
  "UpDyadHi_PBS"
  "UpDyadLo_PBS"
  "DownDyadHi_PBS"
  "DownDyadLo_PBS"
  "UpDyad_SBS"
  "DownDyad_SBS"
  "symm_aconv"
  "symm_iconv"
  "MirrorSymmFilt"
  "extend"
  "dyadpartition"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//


// Generate the ambiguity help
mprintf("Updating Wavelab/DeNoising\n");
helpdir = fullfile(cwd,"DeNoising");
funmat = [
  "WaveShrink"
  "WPDeNoise"
  "CPDeNoise"
  "ThreshCP"
  "ThreshWP"
  "ThreshWave"
  "ThreshWave2"
  "CohWave"
  "IdealWavDenoise"
  "TIDenoiseHard2"
  "TIDenoiseSoft2"
  "MultiHybrid"
  "MultiMAD"
  "MultiSURE"
  "MultiVisu"
  "InvShrink"
  "HybridThresh"
  "MinMaxThresh"
  "SUREThresh"
  "ValSUREThresh"
  "VisuThresh"
  "SoftThresh"
  "HardThresh"
  "NormNoise"
  "GWN"
  "GWN2"
  "GWNoisy"
  "GWNoisy2"
  "HT"
  "HT2"
  "ST"
  "ST2"
  "Wiener"
  "Wiener2"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//



// Generate the ambiguity help
mprintf("Updating Wavelab/Packets/One-D\n");
helpdir = fullfile(cwd,"Packets_One_D");
funmat = [
  "WPAnalysis"
  "WPSynthesis"
  "WPTour"
  "CPAnalysis"
  "CPSynthesis"
  "CPTour"
  "BestBasis"
  "CalcStatTree"
  "PlotBasisTree"
  "PlotPacketTable"
  "ImagePhasePlane"
  "PlotPhasePlane"
  "CompareStdBases"
  "ImageGaborPhase"
  "ImagePhaseVarious"
  "PlotCoeffComparison"
  "PlotCompressNumbers"
  "PlotGaborPhase"
  "PlotPhaseVarious"
  "PlotWavePhase"
  "FPT_WP"
  "IPT_WP"
  "FPT_CP"
  "IPT_CP"
  "MakeCosinePacket"
  "MakeWaveletPacket"
  "fold"
  "edgefold"
  "unfold"
  "edgeunfold"
  "MakeONBell"
  "dct_iv"
  "CalcWPLocation"
  "node"
  "packet"
  "MakeBasis"
  "PackBasisCoeff"
  "UnpackBasisCoeff"
  "UnpackStatValue"
  "CalcTreeHeight"
  "DrawHeisenberg"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//


// Generate the ambiguity help
mprintf("Updating Wavelab/Orthogonal\n");
helpdir = fullfile(cwd,"Orthogonal");
funmat = [
  "FWT_PO"
  "IWT_PO"
  "FWT_CDJV"
  "IWT_CDJV"
  "FWT_IO"
  "IWT_IO"
  "FWT2_PO"
  "IWT2_PO"
  "FTWT2_PO"
  "ITWT2_PO"
  "ContourMultiRes"
  "DisplayMutltiRes"
  "DisplayWaveCoeff"
  "PlotMultiRes"
  "PlotWaveCoeff"
  "MakeCDJVFilter"
  "MakeONFilter"
  "MakeOBFilter"
  "MakeWavelet"
  "Make2dWavelet"
  "UpDyadHi"
  "UpDyadLo"
  "DownDyadHi"
  "DownDyadLo"
  "CDJVDyadDown"
  "CDJVDyadUp"
  "aconv"
  "iconv"
  "dyad"
  "dyad2ix"
  "dyadlength"
  "quad2ix"
  "quadlength"
  "rshift"
  "lshift"
  "MirrorFilt"
  "reverse"
  "UpSampleN"
  "PlotSpikes"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//


// Generate the ambiguity help
mprintf("Updating Wavelab/Meyer\n");
helpdir = fullfile(cwd,"Meyer");
funmat = [
  "FWT_YM"
  "IWT_YM"
  "FWT2_YM"
  "FTWT2_YM"
  "IWT2_YM"
  "ITWT2_YM"
  "PlotYMMultiRes"
  "CoarseMeyerCoeff"
  "DetailMeyerCoeff"
  "FineMeyerCoeff"
  "DetailMeyerProj"
  "CoarseMeyerProj"
  "FineMeyerProj"
  "CombineCoeff"
  "SeparateCoeff"
  "FoldMeyer"
  "UnfoldMeyer"
  "ExtendProj"
  "WindowMeyer"
  "QuasiDCT"
  "dct_ii"
  "dct_iii"
  "QuasiDST"
  "dst_ii"
  "dst_iii"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//




// Generate the ambiguity help
mprintf("Updating Wavelab/Interpolating\n");
helpdir = fullfile(cwd,"Interpolating");
funmat = [
  "DDRefine"
  "MakeDDFilter"
  "MakeDDBdryFilter"
  "FWT_DD"
  "IWT_DD"
  "PlotDDMultiRes"
  "DDDyadDown"
  "DDDyadUp"
  "AIRefine"
  "AIRefine2d"
  "MakeAIFilter"
  "MakeAIBdryFilter"
  "FWT_AI"
  "IWT_AI"
  "PlotAIMultiRes"
  "AIDyadDown"
  "AIDyadUp"
  "HCoarsen2d"
  "HRefine2d"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//






// Generate the ambiguity help
mprintf("Updating Wavelab/Median\n");
helpdir = fullfile(cwd,"Median");
funmat = [
  "FMIPT"
  "IMIPT"
  "PlotMRA_MT"
  "PlotMTCoeff"
  "MedITriadDown"
  "MedITriadUp"
  "QuadMedRef"
  "TriMergeSort"
  "quadmedian"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//




// Generate the ambiguity help
mprintf("Updating Wavelab/FastAlgorithms\n");
helpdir = fullfile(cwd,"FastAlgorithms");
funmat = [
  "FNWT_PO"
  "FSFT_PO"
  "INWT_PO"
  "ISFT_PO"
  "Mat2SparseNSForm"
  "Mat2SparseSForm"
  "ndyad"
  "NonStdWaveMult"
  "StdWaveMult"
  "StretchMatrix"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//



// Generate the ambiguity help
mprintf("Updating Wavelab/Packets_Two_D\n");
helpdir = fullfile(cwd,"Packets_Two_D");
funmat = [
  "WP2dTour"
  "WP2Tour"
  "FPT2_WP"
  "IPT2_WP"
  "CP2dTour"
  "CP2Tour"
  "FPT2_CP"
  "IPT2_CP"
  "Best2dBasis"
  "Calc2dStatTree"
  "Calc2dSQTree"
  "Plot2dBasisTree"
  "Plot2dPartition"
  "Make2dWaveletPacket"
  "Make2dCosinePacket"
  "UpQuad"
  "DownQuad"
  "dct2_iv"
  "qnode"
  "quadbounds"
  "qpkt2ix"
  "BuildFBITree"
  "Calc2dPktTable"
  "Unpack2dBasisCoef"
  "Calc2dTreeHeight"
  "CalcEntropy"
  ];
macrosdir = cwd +"../../macros";
//demosdir = cwd +"../../demos";
demosdir = [];
modulename = "Wavelab";
helpupdate ( funmat , helpdir , macrosdir , demosdir , modulename , %t )
//